package org.basex.io.serial;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import org.basex.data.Data;
import org.basex.data.FTPos;
import org.basex.data.FTPosData;
import org.basex.util.list.IntList;
import org.basex.util.list.TokenList;

public class XMLExtractSerializer extends XMLSerializer {
  protected List<Integer> previousParents = new ArrayList<Integer>();
  protected List<Integer> parents = new ArrayList<Integer>();

  public XMLExtractSerializer(OutputStream os, SerializerProp p) throws IOException {
    super(os, p);
  }


  /**
   * Serializes a node of the specified data reference.
   * @param data data reference
   * @param pre pre value to start from
   * @param ft full-text data
   * @return last pre value
   * @throws IOException I/O exception
   */
  public int node(final Data data, final int pre, final FTPosData ft)
      throws IOException {

    boolean doc = false;
    
    final TokenList nsp = data.ns.size() != 0 ? new TokenList() : null;
    final IntList pars = new IntList();
    int l = 0;
    int p = pre;

    // loop through all table entries
    final int s = pre + data.size(pre, data.kind(p));
//    while(p < s && !finished()) {
      final int k = data.kind(p);
      final int r = data.parent(p, k);

      if(k == Data.DOC) {
        if(doc) closeDoc();
        openDoc(data.text(p++, true));
        doc = true;
      } else if(k == Data.TEXT) {
        final FTPos ftd = ft != null ? ft.get(data, p) : null;
//        if(ftd != null) text(data.text(p++, true), ftd);
//        else 
        text(data.text(p++, true));
      } else if(k == Data.COMM) {
        comment(data.text(p++, true));
      } else if(k == Data.ATTR) {
        attribute(data.name(p, k), data.text(p++, false));
      } else if(k == Data.PI) {
        pi(data.name(p, k), data.atom(p++));
      } else {
        // add element node
        int parentStart = data.parent(p, k);
        l = serializeParents(parentStart, data, l + 1);
        final byte[] name = data.name(p, k);
        openElement(name);

        final int as = p + data.attSize(p, k);
        while(++p != as) {
          attribute(data.name(p, Data.ATTR), data.text(p, false));
        }
        closeElement();
        pars.set(l++, r);
      }
//    }

    List<Integer> swapList;
    parents.addAll(previousParents);
    previousParents.clear();
    swapList = previousParents;
    previousParents = parents;
    parents = swapList;
    if(doc) closeDoc();
    return s;
  }

  private int serializeParents(int p, Data data, int l) throws IOException {
    final int k = data.kind(p);
    /* stop if there are no more parents */
    if(k != Data.ELEM) {
      int idx = previousParents.size();
      for(int i = 0; i < idx; ++i) {
        closeElement();
      }
      previousParents.clear();
      return l - 1;
    }
    /* stop to the first common parent with the previous result */
    if(previousParents.contains(p)) {
      int idx = previousParents.indexOf(p);
      for(int i = 0; i < idx; ++i) {
        previousParents.remove(0);
        closeElement();
      }

      return l - 1;
    }
    parents.add(p);
    if(p > 1) l = serializeParents(data.parent(p, k), data, l + 1);
    final byte[] name = data.name(p, k);
    openElement(name);
    final int as = p + data.attSize(p, k);
    while(++p != as) {
      attribute(data.name(p, Data.ATTR), data.text(p, false));
    }
    return l;
  }

  @Override
  public void close() throws IOException {
    super.close();
    for( int i = 0; i < previousParents.size(); ++i) {
      closeElement();
    }
    previousParents.clear();
    parents.clear();
    out.flush();
  }
 
}
